@@ -31,7 +31,7 @@
 //! ```toml
 //! cortex-m = "0.7.3"
 //! cortex-m-rt = "0.6.13"
-//! stm32-hal2 = { version = "^1.0.1", features = ["l4x3", "l4rt"]}
+//! stm32-hal2 = { version = "^1.3.2", features = ["l4x3", "l4rt"]}
 //! ```
 //!
 //! If you need `embedded-hal` traits, include the `embedded-hal` feature.
@@ -403,10 +403,12 @@ pub mod dfsdm;
 // todo: G0 missing many DMA registers like CCR?
 // todo: F4 needs some mods. So, only working on L4 and G4.
 // todo: L5 has a PAC bug on CCR registers past 1.
+// https://github.com/stm32-rs/stm32-rs/issues/551
 #[cfg(not(any(feature = "f4", feature = "l5")))]
 pub mod dma;
 
-#[cfg(not(any(feature = "h747cm4", feature = "h747cm7")))] // PAC error on bank 2 accessor for H747cmx.
+// #[cfg(not(any(feature = "h747cm4", feature = "h747cm7")))]
+// PAC error on bank 2 accessor for H747cmx.
 pub mod flash;
 
 // todo: PAC doesn't yet support these newer H7 MCUs that use FMAC.
@@ -436,6 +438,9 @@ pub mod ipcc;
 
 pub mod low_power;
 
+#[cfg(any(feature = "h747cm4", feature = "h747cm7"))]
+pub mod power;
+
 // F3, F4, L5, G0, and WL don't have Quad SPI.
 #[cfg(not(any(
 feature = "f3",
