@@ -168,10 +168,6 @@ impl Rtc {
                 }
             }
 
-            // Reset the backup domain.
-            rcc.bdcr.modify(|_, w| w.bdrst().set_bit());
-            rcc.bdcr.modify(|_, w| w.bdrst().clear_bit());
-
             // Set up the LSI or LSE as required.
             match config.clock_source {
                 RtcClockSource::Lsi => {
@@ -196,11 +192,11 @@ impl Rtc {
                 _ => (),
             }
 
-            // 3. Select the RTC clock source in the Backup domain control register (RCC_BDCR).
-            // 4. Enable the RTC clock by setting the RTCEN [15] bit in the Backup domain control
-            // register (RCC_BDCR)
             rcc.bdcr.modify(|_, w| {
+                // 3. Select the RTC clock source in the Backup domain control register (RCC_BDCR).
                 unsafe { w.rtcsel().bits(result.config.clock_source as u8) };
+                // 4. Enable the RTC clock by setting the RTCEN [15] bit in the Backup domain control
+                // register (RCC_BDCR)
                 w.rtcen().set_bit()
             });
         });
